@@ -11,18 +11,42 @@ __all__ = ['TestUtil']
 
 import logging
 import os
-from StringIO import StringIO
+import StringIO
 import sys
 
 import cssutils
 
+PY2x = sys.version_info < (3,0)
+
+class OutReplacement(object):
+    "io.StringIO does not work somehow?!"
+    def __init__(self):
+        self.t = ''
+        
+    def write(self, t):
+        if not PY2x:
+            # py3 hack
+            if t.startswith('b') and t.endswith("'"):
+                t = t[2:-1]
+                
+        self.t += t
+        
+    def getvalue(self):
+        if not PY2x:
+            # py3 hack
+            self.t = self.t.replace('\\n', '\n')
+            self.t = self.t.replace('\\\\', '\\')
+            
+        return self.t
+    
+
 class TestUtil(object):
     def __init__(self):
         "init out and err to catch both"
-        self._out = StringIO()
+        self._out = OutReplacement()#StringIO()
         sys.stdout = self._out
         
-        self._err = StringIO()
+        self._err = StringIO.StringIO()
         hdlr = logging.StreamHandler(self._err)
         cssutils.log._log.addHandler(hdlr)
                 
@@ -35,6 +59,7 @@ class TestUtil(object):
 
         ok = (out == expo and err == expe)
         if not ok:
+            print
             if out != expo:
                 print '### out:\n%r\n### != expout:\n%r\n' % (out, expo)
             else:
@@ -44,7 +69,7 @@ class TestUtil(object):
 modules = 0
 errors = 0
 
-def testmod(module):
+def mod(module):
     global modules, errors
     modules += 1
     t = TestUtil()
@@ -53,30 +78,47 @@ def testmod(module):
     if not ok:
         errors += 1
     print '---', ok, ':', os.path.basename(module.__file__)
+    print 
           
 def main():
-    print "DOCTESTS:::::::::::::"
-    # doctests
-    import website
-    import doctest
-    doctest.testmod(website)
-    
+    if PY2x:
+        print "DOCTESTS:::::::::::::"
+        # doctests
+        import website
+        import doctest
+        doctest.testmod(website)
+        print 
+        print 80*'-' 
+        print 
+        print 
+
     global modules, errors
     
     import build
-    testmod(build)
-    import cssencodings
-    testmod(cssencodings)
+    mod(build)
     import customlog
-    testmod(customlog)
+    mod(customlog)
+    import imports
+    mod(imports) 
     import parse
-    testmod(parse)
+    mod(parse)
     import selectors_tolower
-    testmod(selectors_tolower)
+    mod(selectors_tolower)
+
+    import cssencodings
+    mod(cssencodings)
+    
+    import minify
+    mod(minify)
     
     print 
-    print 70*'-' 
-    print 'Ran %i tests (%i errors).' % (modules, errors)  
+    print 80*'-' 
+    print 'Ran %i tests (%i errors).' % (modules, errors)
+    if PY2x:
+        print '**Check doctest results above!**'
+    else:
+        print 'doctests do not work in Python 3 (yet?)!'
+          
     
 if __name__ == '__main__':
     main()
