@@ -1,36 +1,24 @@
 """CSSComment is not defined in DOM Level 2 at all but a cssutils defined
 class only.
-Implements CSSRule which is also extended for a CSSComment rule type
+
+Implements CSSRule which is also extended for a CSSComment rule type.
 """
 __all__ = ['CSSComment']
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: csscomment.py 1170 2008-03-20 17:42:07Z cthedot $'
+__version__ = '$Id$'
 
-import xml.dom
 import cssrule
 import cssutils
+import xml.dom
 
 class CSSComment(cssrule.CSSRule):
     """
-    (cssutils) a CSS comment
+    Represents a CSS comment (cssutils only).
 
-    Properties
-    ==========
-    cssText: of type DOMString
-        The comment text including comment delimiters
-
-    Inherits properties from CSSRule
-
-    Format
-    ======
-    ::
+    Format::
 
         /*...*/
     """
-    type = property(lambda self: cssrule.CSSRule.COMMENT) # value = -1
-    # constant but needed:
-    wellformed = True 
-
     def __init__(self, cssText=None, parentRule=None, 
                  parentStyleSheet=None, readonly=False):
         super(CSSComment, self).__init__(parentRule=parentRule, 
@@ -42,28 +30,36 @@ class CSSComment(cssrule.CSSRule):
 
         self._readonly = readonly
 
+    def __repr__(self):
+        return u"cssutils.css.%s(cssText=%r)" % (
+                self.__class__.__name__, 
+                self.cssText)
+
+    def __str__(self):
+        return u"<cssutils.css.%s object cssText=%r at 0x%x>" % (
+                self.__class__.__name__, 
+                self.cssText, 
+                id(self))
+
     def _getCssText(self):
-        """returns serialized property cssText"""
+        """Return serialized property cssText."""
         return cssutils.ser.do_CSSComment(self)
 
     def _setCssText(self, cssText):
         """
-        cssText
+        :param cssText:
             textual text to set or tokenlist which is not tokenized
             anymore. May also be a single token for this rule
-        parser
-            if called from cssparser directly this is Parser instance
 
-        DOMException on setting
-
-        - SYNTAX_ERR: (self)
-          Raised if the specified CSS string value has a syntax error and
-          is unparsable.
-        - INVALID_MODIFICATION_ERR: (self)
-          Raised if the specified CSS string value represents a different
-          type of rule than the current one.
-        - NO_MODIFICATION_ALLOWED_ERR: (CSSRule)
-          Raised if the rule is readonly.
+        :exceptions:
+            - :exc:`~xml.dom.SyntaxErr`:
+              Raised if the specified CSS string value has a syntax error and
+              is unparsable.
+            - :exc:`~xml.dom.InvalidModificationErr`:
+              Raised if the specified CSS string value represents a different
+              type of rule than the current one.
+            - :exc:`~xml.dom.NoModificationAllowedErr`:
+              Raised if the rule is readonly.
         """
         super(CSSComment, self)._setCssText(cssText)
         tokenizer = self._tokenize2(cssText)
@@ -81,12 +77,11 @@ class CSSComment(cssrule.CSSRule):
             self._cssText = self._tokenvalue(commenttoken)
 
     cssText = property(_getCssText, _setCssText,
-        doc=u"(cssutils) Textual representation of this comment")
+        doc=u"The parsable textual representation of this rule.")
 
-    def __repr__(self):
-        return "cssutils.css.%s(cssText=%r)" % (
-                self.__class__.__name__, self.cssText)
-
-    def __str__(self):
-        return "<cssutils.css.%s object cssText=%r at 0x%x>" % (
-                self.__class__.__name__, self.cssText, id(self))
+    type = property(lambda self: self.COMMENT, 
+                    doc=u"The type of this rule, as defined by a CSSRule "
+                        u"type constant.")
+    
+    # constant but needed:
+    wellformed = property(lambda self: True)
