@@ -35,7 +35,7 @@ style sheet had been::
     IMG { }
     IMG { }
 
-Cssutils again will issue a message (WARNING in this case) about invalid 
+Cssutils again will issue a message (WARNING in this case) about invalid
 CSS2 property values.
 
 TODO:
@@ -91,10 +91,11 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             <BLANKLINE>
 
     Format::
-    
+
         [Property: Value Priority?;]* [Property: Value Priority?]?
     """
-    def __init__(self, cssText=u'', parentRule=None, readonly=False):
+    def __init__(self, cssText=u'', parentRule=None, readonly=False,
+                 validating=None):
         """
         :param cssText:
             Shortcut, sets CSSStyleDeclaration.cssText
@@ -103,15 +104,19 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             None if this CSSStyleDeclaration is not attached to a CSSRule.
         :param readonly:
             defaults to False
+        :param validating:
+            a flag defining if this sheet should be validated on change.
+            Defaults to None, which means defer to the parent stylesheet.
         """
         super(CSSStyleDeclaration, self).__init__()
         self._parentRule = parentRule
+        self.validating = validating
         self.cssText = cssText
         self._readonly = readonly
 
     def __contains__(self, nameOrProperty):
         """Check if a property (or a property with given name) is in style.
-        
+
         :param name:
             a string or Property, uses normalized name and not literalname
         """
@@ -120,7 +125,7 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         else:
             name = self._normalize(nameOrProperty)
         return name in self.__nnames()
-    
+
     def __iter__(self):
         """Iterator of set Property objects with different normalized names."""
         def properties():
@@ -132,18 +137,18 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         """Analoguous to standard dict returns property names which are set in
         this declaration."""
         return list(self.__nnames())
-    
+
     def __getitem__(self, CSSName):
         """Retrieve the value of property ``CSSName`` from this declaration.
-        
+
         ``CSSName`` will be always normalized.
         """
         return self.getPropertyValue(CSSName)
-    
+
     def __setitem__(self, CSSName, value):
-        """Set value of property ``CSSName``. ``value`` may also be a tuple of 
+        """Set value of property ``CSSName``. ``value`` may also be a tuple of
         (value, priority), e.g. style['color'] = ('red', 'important')
-        
+
         ``CSSName`` will be always normalized.
         """
         priority = None
@@ -154,9 +159,9 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
 
     def __delitem__(self, CSSName):
         """Delete property ``CSSName`` from this declaration.
-        If property is not in this declaration return u'' just like 
+        If property is not in this declaration return u'' just like
         removeProperty.
-        
+
         ``CSSName`` will be always normalized.
         """
         return self.removeProperty(CSSName)
@@ -171,8 +176,8 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         """
         known = ['_tokenizer', '_log', '_ttypes',
                  '_seq', 'seq', 'parentRule', '_parentRule', 'cssText',
-                 'valid', 'wellformed',
-                 '_readonly', '_profiles']
+                 'valid', 'wellformed', 'validating',
+                 '_readonly', '_profiles', '_validating']
         known.extend(CSS2Properties._properties)
         if n in known:
             super(CSSStyleDeclaration, self).__setattr__(n, v)
@@ -195,14 +200,14 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
 
     def __nnames(self):
         """Return iterator for all different names in order as set
-        if names are set twice the last one is used (double reverse!) 
+        if names are set twice the last one is used (double reverse!)
         """
         names = []
         for item in reversed(self.seq):
             val = item.value
             if isinstance(val, Property) and not val.name in names:
                 names.append(val.name)
-        return reversed(names)    
+        return reversed(names)
 
     # overwritten accessor functions for CSS2Properties' properties
     def _getP(self, CSSName):
@@ -236,7 +241,7 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             >>> style.fontStyle = 'italic'
             >>> # or
             >>> style.setProperty('font-style', 'italic', '!important')
-            
+
         """
         self.setProperty(CSSName, value)
         # TODO: Shorthand ones
@@ -282,10 +287,10 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         tokenizer = self._tokenize2(cssText)
 
         # for closures: must be a mutable
-        new = {'wellformed': True} 
+        new = {'wellformed': True}
         def ident(expected, seq, token, tokenizer=None):
             # a property
-            
+
             tokens = self._tokensupto2(tokenizer, starttoken=token,
                                        semicolon=True)
             if self._tokenvalue(tokens[-1]) == u';':
@@ -293,37 +298,46 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             property = Property(parent=self)
             property.cssText = tokens
             if property.wellformed:
-                seq.append(property, 'Property')                
+                seq.append(property, 'Property')
             else:
                 self._log.error(u'CSSStyleDeclaration: Syntax Error in '
                                 u'Property: %s' % self._valuestr(tokens))
             # does not matter in this case
-            return expected 
+            return expected
 
         def unexpected(expected, seq, token, tokenizer=None):
             # error, find next ; or } to omit upto next property
             ignored = self._tokenvalue(token) + self._valuestr(
-                                self._tokensupto2(tokenizer, 
+                                self._tokensupto2(tokenizer,
                                                   propertyvalueendonly=True))
             self._log.error(u'CSSStyleDeclaration: Unexpected token, ignoring '
                             'upto %r.' % ignored,token)
             # does not matter in this case
             return expected
 
+        def char(expected, seq, token, tokenizer=None):
+            # a standalone ; or error...
+            if self._tokenvalue(token) == u';':
+                self._log.info(u'CSSStyleDeclaration: Stripped standalone semicolon'
+                                u': %s' % self._valuestr([token]), neverraise=True)
+                return expected
+            else:
+                return unexpected(expected, seq, token, tokenizer)
+
         # [Property: Value;]* Property: Value?
         newseq = self._tempSeq()
         wellformed, expected = self._parse(expected=None,
             seq=newseq, tokenizer=tokenizer,
-            productions={'IDENT': ident},#, 'CHAR': char},
+            productions={'IDENT': ident, 'CHAR': char},
             default=unexpected)
         # wellformed set by parse
 
         for item in newseq:
             item.value._parent = self
-         
-        # do not check wellformed as invalid things are removed anyway            
+
+        # do not check wellformed as invalid things are removed anyway
         self._setSeq(newseq)
-        
+
     cssText = property(_getCssText, _setCssText,
                        doc=u"(DOM) A parsable textual representation of the "
                            u"declaration block excluding the surrounding curly "
@@ -331,7 +345,7 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
 
     def getCssText(self, separator=None):
         """
-        :returns: 
+        :returns:
             serialized property cssText, each property separated by
             given `separator` which may e.g. be ``u''`` to be able to use
             cssText directly in an HTML style attribute. ``;`` is part of
@@ -344,7 +358,7 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         self._parentRule = parentRule
 #        for x in self.children():
 #            x.parent = self
-            
+
     parentRule = property(lambda self: self._parentRule, _setParentRule,
         doc="(DOM) The CSS rule that contains this declaration block or "
             "None if this CSSStyleDeclaration is not attached to a CSSRule.")
@@ -354,18 +368,18 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         :param name:
             optional `name` of properties which are requested.
             Only properties with this **always normalized** `name` are returned.
-            If `name` is ``None`` all properties are returned (at least one for 
+            If `name` is ``None`` all properties are returned (at least one for
             each set name depending on parameter `all`).
         :param all:
-            if ``False`` (DEFAULT) only the effective properties are returned. 
+            if ``False`` (DEFAULT) only the effective properties are returned.
             If name is given a list with only one property is returned.
 
             if ``True`` all properties including properties set multiple times
             with different values or priorities for different UAs are returned.
-            The order of the properties is fully kept as in the original 
+            The order of the properties is fully kept as in the original
             stylesheet.
-        :returns: 
-            a list of :class:`~cssutils.css.Property` objects set in 
+        :returns:
+            a list of :class:`~cssutils.css.Property` objects set in
             this declaration.
         """
         if name and not all:
@@ -373,13 +387,13 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             p = self.getProperty(name)
             if p:
                 return [p]
-            else: 
+            else:
                 return []
         elif not all:
             # effective Properties in name order
-            return [self.getProperty(name)for name in self.__nnames()]
-        else:    
-            # all properties or all with this name    
+            return [self.getProperty(name) for name in self.__nnames()]
+        else:
+            # all properties or all with this name
             nname = self._normalize(name)
             properties = []
             for item in self.seq:
@@ -397,7 +411,7 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             if ``True`` (DEFAULT) name will be normalized (lowercase, no simple
             escapes) so "color", "COLOR" or "C\olor" will all be equivalent
 
-            If ``False`` may return **NOT** the effective value but the 
+            If ``False`` may return **NOT** the effective value but the
             effective for the unnormalized name.
         :returns:
             the effective :class:`~cssutils.css.Property` object.
@@ -422,23 +436,23 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             if ``True`` (DEFAULT) name will be normalized (lowercase, no simple
             escapes) so "color", "COLOR" or "C\olor" will all be equivalent
 
-            If ``False`` may return **NOT** the effective value but the 
+            If ``False`` may return **NOT** the effective value but the
             effective for the unnormalized name.
         :returns:
             :class:`~cssutils.css.CSSValue`, the value of the effective
-            property if it has been explicitly set for this declaration block. 
+            property if it has been explicitly set for this declaration block.
 
         (DOM)
         Used to retrieve the object representation of the value of a CSS
         property if it has been explicitly set within this declaration
         block. Returns None if the property has not been set.
-                
+
         (This method returns None if the property is a shorthand
         property. Shorthand property values can only be accessed and
         modified as strings, using the getPropertyValue and setProperty
         methods.)
 
-        **cssutils currently always returns a CSSValue if the property is 
+        **cssutils currently always returns a CSSValue if the property is
         set.**
 
         for more on shorthand properties see
@@ -447,15 +461,15 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         nname = self._normalize(name)
         if nname in self._SHORTHANDPROPERTIES:
             self._log.info(u'CSSValue for shorthand property "%s" should be '
-                           u'None, this may be implemented later.' % 
+                           u'None, this may be implemented later.' %
                            nname, neverraise=True)
 
         p = self.getProperty(name, normalize)
         if p:
-            return p.cssValue
+            return p.propertyValue
         else:
             return None
-        
+
     def getPropertyValue(self, name, normalize=True):
         """
         :param name:
@@ -476,7 +490,7 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             return p.value
         else:
             return u''
-        
+
     def getPropertyPriority(self, name, normalize=True):
         """
         :param name:
@@ -512,8 +526,8 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             The effective Property value is returned and *all* Properties
             with ``Property.name == name`` are removed.
 
-            If ``False`` may return **NOT** the effective value but the 
-            effective for the unnormalized `name` only. Also only the 
+            If ``False`` may return **NOT** the effective value but the
+            effective for the unnormalized `name` only. Also only the
             Properties with the literal name `name` are removed.
         :returns:
             the value of the property if it has been explicitly set for
@@ -534,19 +548,20 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             # remove all properties with name == nname
             nname = self._normalize(name)
             for item in self.seq:
-                if not (isinstance(item.value, Property) 
+                if not (isinstance(item.value, Property)
                         and item.value.name == nname):
                     newseq.appendItem(item)
         else:
             # remove all properties with literalname == name
             for item in self.seq:
-                if not (isinstance(item.value, Property) 
+                if not (isinstance(item.value, Property)
                         and item.value.literalname == name):
                     newseq.appendItem(item)
         self._setSeq(newseq)
         return r
 
-    def setProperty(self, name, value=None, priority=u'', normalize=True):
+    def setProperty(self, name, value=None, priority=u'',
+                    normalize=True, replace=True):
         """(DOM) Set a property value and priority within this declaration
         block.
 
@@ -555,11 +570,11 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
             "propertyName"), always lowercase (even if not normalized)
 
             If a property with this `name` is present it will be reset.
-            
-            cssutils also allowed `name` to be a 
+
+            cssutils also allowed `name` to be a
             :class:`~cssutils.css.Property` object, all other
             parameter are ignored in this case
-        
+
         :param value:
             the new value of the property, ignored if `name` is a Property.
         :param priority:
@@ -568,6 +583,17 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
         :param normalize:
             if True (DEFAULT) `name` will be normalized (lowercase, no simple
             escapes) so "color", "COLOR" or "C\olor" will all be equivalent
+        :param replace:
+            if True (DEFAULT) the given property will replace a present
+            property. If False a new property will be added always.
+            The difference to `normalize` is that two or more properties with
+            the same name may be set, useful for e.g. stuff like::
+
+                background: red;
+                background: rgba(255, 0, 0, 0.5);
+
+            which defines the same property but only capable UAs use the last
+            property value, older ones use the first value.
 
         :exceptions:
             - :exc:`~xml.dom.SyntaxErr`:
@@ -578,35 +604,40 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
               readonly.
         """
         self._checkReadonly()
-        
+
         if isinstance(name, Property):
-            newp = name 
+            newp = name
             name = newp.literalname
         elif not value:
             # empty string or None effectively removed property
             return self.removeProperty(name)
         else:
             newp = Property(name, value, priority)
-        if not newp.wellformed:
+
+        if newp.wellformed:
+            if replace:
+                # check if update
+                nname = self._normalize(name)
+                properties = self.getProperties(name, all=(not normalize))
+                for property in reversed(properties):
+                    if normalize and property.name == nname:
+                        property.propertyValue = newp.propertyValue.cssText
+                        property.priority = newp.priority
+                        return
+                    elif property.literalname == name:
+                        property.propertyValue = newp.propertyValue.cssText
+                        property.priority = newp.priority
+                        return
+
+            # not yet set or forced omit replace
+            newp.parent = self
+            self.seq._readonly = False
+            self.seq.append(newp, 'Property')
+            self.seq._readonly = True
+
+        else:
             self._log.warn(u'Invalid Property: %s: %s %s'
                            % (name, value, priority))
-        else:
-            nname = self._normalize(name)
-            properties = self.getProperties(name, all=(not normalize))
-            for property in reversed(properties):
-                if normalize and property.name == nname:
-                    property.cssValue = newp.cssValue.cssText
-                    property.priority = newp.priority
-                    break
-                elif property.literalname == name:
-                    property.cssValue = newp.cssValue.cssText
-                    property.priority = newp.priority
-                    break
-            else:
-                newp.parent = self
-                self.seq._readonly = False
-                self.seq.append(newp, 'Property')
-                self.seq._readonly = True
 
     def item(self, index):
         """(DOM) Retrieve the properties that have been explicitly set in
@@ -643,3 +674,24 @@ class CSSStyleDeclaration(CSS2Properties, cssutils.util.Base2):
                           u"These are properties with a different ``name`` "
                           u"only. :meth:`item` and :attr:`length` work on the "
                           u"same set here.")
+
+    def _getValidating(self):
+        try:
+            # CSSParser.parseX() sets validating of stylesheet
+            return self.parentRule.parentStyleSheet.validating
+        except AttributeError:
+            # CSSParser.parseStyle() sets validating of declaration
+            if self._validating is not None:
+                return self._validating
+        # default
+        return True
+
+    def _setValidating(self, validating):
+        self._validating = validating
+
+    validating = property(_getValidating, _setValidating,
+                          doc=u"If ``True`` this declaration validates "
+                          u"contained properties. The parent StyleSheet "
+                          u"validation setting does *always* win though so "
+                          u"even if validating is True it may not validate "
+                          u"if the StyleSheet defines else!")
