@@ -2,41 +2,149 @@
 """
 __all__ = []
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: util.py 1395 2008-07-27 13:30:13Z cthedot $'
+__version__ = '$Id$'
 
-import codecs
-from itertools import ifilter
-import re
-import types
-import urllib2
-import xml.dom
+from helper import normalize
+from itertools import ifilter, chain
 import cssutils
-from tokenize2 import Tokenizer
-# COMMENT OUT IF RUNNING THIS TEST STANDALONE!
-import encutils
+import codec
+import codecs
+import errorhandler
+import tokenize2
+import types
+import xml.dom
 
-class Base(object):
+try:
+    from _fetchgae import _defaultFetcher
+except ImportError, e:
+    from _fetch import _defaultFetcher
+
+log = errorhandler.ErrorHandler()
+
+class _BaseClass(object):
     """
-    Base class for most CSS and StyleSheets classes
+    Base class for Base, Base2 and _NewBase.
+
+    **Base and Base2 will be removed in the future!**
+    """
+    _log = errorhandler.ErrorHandler()
+    _prods = tokenize2.CSSProductions
+
+    def _checkReadonly(self):
+        "Raise xml.dom.NoModificationAllowedErr if rule/... is readonly"
+        if hasattr(self, '_readonly') and self._readonly:
+            raise xml.dom.NoModificationAllowedErr(
+                u'%s is readonly.' % self.__class__)
+            return True
+        return False
+
+    def _valuestr(self, t):
+        """
+        Return string value of t (t may be a string, a list of token tuples
+        or a single tuple in format (type, value, line, col).
+        Mainly used to get a string value of t for error messages.
+        """
+        if not t:
+            return u''
+        elif isinstance(t, basestring):
+            return t
+        else:
+            return u''.join([x[1] for x in t])
+
+
+class _NewBase(_BaseClass):
+    """
+    New base class for classes using ProdParser.
+
+    **Currently CSSValue and related ones only.**
+    """
+    def __init__(self):
+        self._seq = Seq()
+
+    def _setSeq(self, newseq):
+        """Set value of ``seq`` which is readonly."""
+        newseq._readonly = True
+        self._seq = newseq
+
+    def _clearSeq(self):
+        self._seq.clear()
+
+    def _tempSeq(self, readonly=False):
+        "Get a writeable Seq() which is used to set ``seq`` later"
+        return Seq(readonly=readonly)
+
+    seq = property(lambda self: self._seq,
+                   doc="Internal readonly attribute, **DO NOT USE**!")
+
+
+class _NewListBase(_NewBase):
+    """
+    (EXPERIMENTAL)
+    A base class used for list classes like stylesheets.MediaList
+
+    adds list like behaviour running on inhering class' property ``seq``
+
+    - item in x => bool
+    - len(x) => integer
+    - get, set and del x[i]
+    - for item in x
+    - append(item)
+
+    some methods must be overwritten in inheriting class
+    """
+    def __init__(self):
+        self._seq = Seq()
+
+    def __contains__(self, item):
+        for it in self._seq:
+            if item == it.value:
+                return True
+        return False
+
+    def __delitem__(self, index):
+        del self._seq[index]
+
+    def __getitem__(self, index):
+        return self._seq[index].value
+
+    def __iter__(self):
+        def gen():
+            for x in self._seq:
+                yield x.value
+        return gen()
+
+    def __len__(self):
+        return len(self._seq)
+
+    def __setitem__(self, index, item):
+        "must be overwritten"
+        raise NotImplementedError
+
+    def append(self, item):
+        "must be overwritten"
+        raise NotImplementedError
+
+
+class Base(_BaseClass):
+    """
+    **Superceded by _NewBase**
 
     **Superceded by Base2 which is used for new seq handling class.**
-    See cssutils.util.Base2
+
+    Base class for most CSS and StyleSheets classes
 
     Contains helper methods for inheriting classes helping parsing
 
     ``_normalize`` is static as used by Preferences.
     """
-    __tokenizer2 = Tokenizer()
-
-    _log = cssutils.log
-    _prods = cssutils.tokenize2.CSSProductions
+    __tokenizer2 = tokenize2.Tokenizer()
 
     # for more on shorthand properties see
     # http://www.dustindiaz.com/css-shorthand/
     # format: shorthand: [(propname, mandatorycheck?)*]
     _SHORTHANDPROPERTIES = {
             u'background': [],
-            u'background-position': [],
+            #u'background-position': [], # list of 2 values!
             u'border': [],
             u'border-left': [],
             u'border-right': [],
@@ -54,9 +162,6 @@ class Base(object):
             u'pause': []
             }
 
-    # simple escapes, all non unicodes
-    __simpleescapes = re.compile(ur'(\\[^0-9a-fA-F])').sub
-
     @staticmethod
     def _normalize(x):
         """
@@ -67,21 +172,7 @@ class Base(object):
           been resolved by the tokenizer already)
         - lowercase
         """
-        if x:
-            def removeescape(matchobj):
-                return matchobj.group(0)[1:]
-            x = Base.__simpleescapes(removeescape, x)
-            return x.lower()
-        else:
-            return x
-
-    def _checkReadonly(self):
-        "raises xml.dom.NoModificationAllowedErr if rule/... is readonly"
-        if hasattr(self, '_readonly') and self._readonly:
-            raise xml.dom.NoModificationAllowedErr(
-                u'%s is readonly.' % self.__class__)
-            return True
-        return False
+        return normalize(x)
 
     def _splitNamespacesOff(self, text_namespaces_tuple):
         """
@@ -92,10 +183,10 @@ class Base(object):
         CSSStyleSheet
         """
         if isinstance(text_namespaces_tuple, tuple):
-            return text_namespaces_tuple[0], _SimpleNamespaces(
+            return text_namespaces_tuple[0], _SimpleNamespaces(self._log,
                                                     text_namespaces_tuple[1])
         else:
-            return text_namespaces_tuple, _SimpleNamespaces()
+            return text_namespaces_tuple, _SimpleNamespaces(log=self._log)
 
     def _tokenize2(self, textortokens):
         """
@@ -108,21 +199,19 @@ class Base(object):
             # needs to be tokenized
             return self.__tokenizer2.tokenize(
                  textortokens)
-        elif types.GeneratorType == type(textortokens):
-            # already tokenized
-            return textortokens
         elif isinstance(textortokens, tuple):
             # a single token (like a comment)
             return [textortokens]
         else:
-            # already tokenized but return generator
-            return (x for x in textortokens)
+            # already tokenized but return an iterator
+            return iter(textortokens)
 
     def _nexttoken(self, tokenizer, default=None):
         "returns next token in generator tokenizer or the default value"
         try:
             return tokenizer.next()
-        except (StopIteration, AttributeError):
+        # TypeError for py3
+        except (StopIteration, AttributeError, TypeError):
             return default
 
     def _type(self, token):
@@ -150,7 +239,7 @@ class Base(object):
         """
         if token:
             value = token[1]
-            return value.replace('\\'+value[0], value[0])[1:-1]
+            return value.replace('\\' + value[0], value[0])[1: - 1]
         else:
             return None
 
@@ -162,10 +251,10 @@ class Base(object):
              url("\"") => "
         """
         if token:
-            value = token[1][4:-1].strip()
-            if (value[0] in '\'"') and (value[0] == value[-1]):
+            value = token[1][4: - 1].strip()
+            if value and (value[0] in '\'"') and (value[0] == value[ - 1]):
                 # a string "..." or '...'
-                value = value.replace('\\'+value[0], value[0])[1:-1]
+                value = value.replace('\\' + value[0], value[0])[1: - 1]
             return value
         else:
             return None
@@ -199,7 +288,7 @@ class Base(object):
 
         if blockstartonly: # {
             ends = u'{'
-            brace = -1 # set to 0 with first {
+            brace = - 1 # set to 0 with first {
         elif blockendonly: # }
             ends = u'}'
             brace = 1
@@ -214,7 +303,7 @@ class Base(object):
             # end of mediaquery which may be { or STRING
             # special case, see below
             ends = u'{'
-            brace = -1 # set to 0 with first {
+            brace = - 1 # set to 0 with first {
             endtypes = ('STRING',)
         elif semicolon:
             ends = u';'
@@ -237,12 +326,21 @@ class Base(object):
         resulttokens = []
         if starttoken:
             resulttokens.append(starttoken)
+            val = starttoken[1]
+            if u'[' == val:
+                bracket += 1
+            elif u'{' == val:
+                brace += 1
+            elif u'(' == val:
+                parant += 1
+
         if tokenizer:
             for token in tokenizer:
                 typ, val, line, col = token
                 if 'EOF' == typ:
                     resulttokens.append(token)
                     break
+
                 if u'{' == val:
                     brace += 1
                 elif u'}' == val:
@@ -263,33 +361,19 @@ class Base(object):
                 if (brace == bracket == parant == 0) and (
                     val in ends or typ in endtypes):
                     break
-                elif mediaqueryendonly and brace == -1 and (
+                elif mediaqueryendonly and brace == - 1 and (
                      bracket == parant == 0) and typ in endtypes:
                      # mediaqueryendonly with STRING
                     break
-
         if separateEnd:
             # TODO: use this method as generator, then this makes sense
             if resulttokens:
-                return resulttokens[:-1], resulttokens[-1]
+                return resulttokens[: - 1], resulttokens[ - 1]
             else:
                 return resulttokens, None
         else:
             return resulttokens
 
-    def _valuestr(self, t):
-        """
-        returns string value of t (t may be a string, a list of token tuples
-        or a single tuple in format (type, value, line, col).
-        Mainly used to get a string value of t for error messages.
-        """
-        if not t:
-            return u''
-        elif isinstance(t, basestring):
-            return t
-        else:
-            return u''.join([x[1] for x in t])
-
     def _adddefaultproductions(self, productions, new=None):
         """
         adds default productions if not already present, used by
@@ -301,7 +385,7 @@ class Base(object):
         the current value of self.__expected
         """
         def ATKEYWORD(expected, seq, token, tokenizer=None):
-            "TODO: add default impl for unexpected @rule?"
+            "default impl for unexpected @rule"
             if expected != 'EOF':
                 # TODO: parentStyleSheet=self
                 rule = cssutils.css.CSSUnknownRule()
@@ -336,7 +420,7 @@ class Base(object):
         return p
 
     def _parse(self, expected, seq, tokenizer, productions, default=None,
-               new=None):
+               new=None, initialtoken=None):
         """
         puts parsed tokens in seq by calling a production with
             (seq, tokenizer, token)
@@ -353,13 +437,28 @@ class Base(object):
             default callback if tokentype not in productions
         new
             used to init default productions
+        initialtoken
+            will be used together with tokenizer running 1st this token
+            and then all tokens in tokenizer
 
         returns (wellformed, expected) which the last prod might have set
         """
         wellformed = True
-        if tokenizer:
+
+        if initialtoken:
+            # add initialtoken to tokenizer
+            def tokens():
+                "Build new tokenizer including initialtoken"
+                yield initialtoken
+                for item in tokenizer:
+                    yield item
+            fulltokenizer = chain([initialtoken], tokenizer)
+        else:
+            fulltokenizer = tokenizer
+
+        if fulltokenizer:
             prods = self._adddefaultproductions(productions, new)
-            for token in tokenizer:
+            for token in fulltokenizer:
                 p = prods.get(token[0], default)
                 if p:
                     expected = p(expected, seq, token, tokenizer)
@@ -369,26 +468,15 @@ class Base(object):
         return wellformed, expected
 
 
-class Base2(Base):
+class Base2(Base, _NewBase):
     """
-    Base class for new seq handling, used by Selector for now only
+    **Superceded by _NewBase.**
+
+    Base class for new seq handling.
     """
     def __init__(self):
         self._seq = Seq()
 
-    def _setSeq(self, newseq):
-        """
-        sets newseq and makes it readonly
-        """
-        newseq._readonly = True
-        self._seq = newseq
-
-    seq = property(lambda self: self._seq, doc="seq for most classes")
-
-    def _tempSeq(self, readonly=False):
-        "get a writeable Seq() which is added later"
-        return Seq(readonly=readonly)
-
     def _adddefaultproductions(self, productions, new=None):
         """
         adds default productions if not already present, used by
@@ -458,6 +546,27 @@ class Seq(object):
         self._seq = []
         self._readonly = readonly
 
+    def __repr__(self):
+        "returns a repr same as a list of tuples of (value, type)"
+        return u'cssutils.%s.%s([\n    %s], readonly=%r)' % (self.__module__,
+                                          self.__class__.__name__,
+            u',\n    '.join([u'%r' % item for item in self._seq]
+            ), self._readonly)
+
+    def __str__(self):
+        vals = []
+        for v in self:
+            if isinstance(v.value, basestring):
+                vals.append(v.value)
+            elif isinstance(v, tuple):
+                vals.append(v.value[1])
+            else:
+                vals.append(repr(v))
+
+        return "<cssutils.%s.%s object length=%r items=%r readonly=%r at 0x%x>" % (
+                self.__module__, self.__class__.__name__, len(self),
+                vals, self._readonly, id(self))
+
     def __delitem__(self, i):
         del self._seq[i]
 
@@ -473,12 +582,15 @@ class Seq(object):
     def __len__(self):
         return len(self._seq)
 
-    def append(self, val, typ, line=None, col=None):
-        "if not readonly add new Item()"
+    def append(self, val, typ=None, line=None, col=None):
+        "If not readonly add new Item()"
         if self._readonly:
             raise AttributeError('Seq is readonly.')
         else:
-            self._seq.append(Item(val, typ, line, col))
+            if isinstance(val, Item):
+                self._seq.append(val)
+            else:
+                self._seq.append(Item(val, typ, line, col))
 
     def appendItem(self, item):
         "if not readonly add item which must be an Item"
@@ -487,6 +599,13 @@ class Seq(object):
         else:
             self._seq.append(item)
 
+    def clear(self):
+        del self._seq[:]
+
+    def insert(self, index, val, typ, line=None, col=None):
+        "Insert new Item() at index # even if readony!? TODO!"
+        self._seq.insert(index, Item(val, typ, line, col))
+    
     def replace(self, index=-1, val=None, typ=None, line=None, col=None):
         """
         if not readonly replace Item at index with new Item or
@@ -497,16 +616,23 @@ class Seq(object):
         else:
             self._seq[index] = Item(val, typ, line, col)
 
-    def __repr__(self):
-        "returns a repr same as a list of tuples of (value, type)"
-        return u'cssutils.%s.%s([\n    %s])' % (self.__module__,
-                                          self.__class__.__name__,
-            u',\n    '.join([u'(%r, %r)' % (item.type, item.value)
-                          for item in self._seq]
-            ))
-    def __str__(self):
-        return "<cssutils.%s.%s object length=%r at 0x%x>" % (
-                self.__module__, self.__class__.__name__, len(self), id(self))
+    def rstrip(self):
+        "trims S items from end of Seq"
+        while self._seq and self._seq[ - 1].type == tokenize2.CSSProductions.S:
+            # TODO: removed S before CSSComment /**/ /**/
+            del self._seq[ - 1]
+
+    def appendToVal(self, val=None, index= - 1):
+        """
+        if not readonly append to Item's value at index
+        """
+        if self._readonly:
+            raise AttributeError('Seq is readonly.')
+        else:
+            old = self._seq[index]
+            self._seq[index] = Item(old.value + val, old.type,
+                                    old.line, old.col)
+
 
 class Item(object):
     """
@@ -542,7 +668,7 @@ class Item(object):
 class ListSeq(object):
     """
     (EXPERIMENTAL)
-    A base class used for list classes like css.SelectorList or
+    A base class used for list classes like cssutils.css.SelectorList or
     stylesheets.MediaList
 
     adds list like behaviour running on inhering class' property ``seq``
@@ -601,9 +727,13 @@ class _Namespaces(object):
     parentStyleSheet
         the parent CSSStyleSheet
     """
-    def __init__(self, parentStyleSheet, *args):
+    def __init__(self, parentStyleSheet, log=None, *args):
         "no initial values are set, only the relevant sheet is"
         self.parentStyleSheet = parentStyleSheet
+        self._log = log
+
+    def __repr__(self):
+        return "%r" % self.namespaces
 
     def __contains__(self, prefix):
         return prefix in self.namespaces
@@ -622,13 +752,15 @@ class _Namespaces(object):
                 self.parentStyleSheet.deleteRule(i)
                 return
 
-        raise xml.dom.NamespaceErr('Prefix %r not found.' % prefix)
+        self._log.error('Prefix %s not found.' % prefix,
+                        error=xml.dom.NamespaceErr)
 
     def __getitem__(self, prefix):
         try:
             return self.namespaces[prefix]
         except KeyError, e:
-            raise xml.dom.NamespaceErr('Prefix %r not found.' % prefix)
+            self._log.error('Prefix %s not found.' % prefix,
+                            error=xml.dom.NamespaceErr)
 
     def __iter__(self):
         return self.namespaces.__iter__()
@@ -659,7 +791,12 @@ class _Namespaces(object):
             if rule.prefix == prefix:
                 return rule
 
-    def __getNamespaces(self):
+    @property
+    def namespaces(self):
+        """
+        A property holding only effective @namespace rules in
+        self.parentStyleSheets.
+        """
         namespaces = {}
         for rule in ifilter(lambda r: r.type == r.NAMESPACE_RULE,
                             reversed(self.parentStyleSheet.cssRules)):
@@ -667,10 +804,6 @@ class _Namespaces(object):
                 namespaces[rule.prefix] = rule.namespaceURI
         return namespaces
 
-    namespaces = property(__getNamespaces,
-        doc=u'Holds only effective @namespace rules in self.parentStyleSheets'
-             '@namespace rules.')
-
     def get(self, prefix, default):
         return self.namespaces.get(prefix, default)
 
@@ -690,7 +823,7 @@ class _Namespaces(object):
         for prefix, uri in self.namespaces.items():
             if uri == namespaceURI:
                 return prefix
-        raise IndexError(u'NamespaceURI %r not found.' % namespaceURI)
+        raise IndexError(u'NamespaceURI %s not found.' % namespaceURI)
 
     def __str__(self):
         return u"<cssutils.util.%s object parentStyleSheet=%r at 0x%x>" % (
@@ -702,7 +835,9 @@ class _SimpleNamespaces(_Namespaces):
     namespaces used in objects like Selector as long as they are not connected
     to a CSSStyleSheet
     """
-    def __init__(self, *args):
+    def __init__(self, log=None, *args):
+        """init"""
+        super(_SimpleNamespaces, self).__init__(parentStyleSheet=None, log=log)
         self.__namespaces = dict(*args)
 
     def __setitem__(self, prefix, namespaceURI):
@@ -720,59 +855,31 @@ class _SimpleNamespaces(_Namespaces):
             self.namespaces)
 
 
-def _defaultFetcher(url):
-    """Retrieve data from ``url``. cssutils default implementation of fetch
-    URL function.
-
-    Returns ``(encoding, string)`` or ``None``
-    """
-    try:
-        res = urllib2.urlopen(url)
-    except OSError, e:
-        # e.g if file URL and not found
-        cssutils.log.warn(e, error=OSError)
-    except (OSError, ValueError), e:
-        # invalid url, e.g. "1"
-        cssutils.log.warn(u'ValueError, %s' % e.message, error=ValueError)
-    except urllib2.HTTPError, e:
-        # http error, e.g. 404, e can be raised
-        cssutils.log.warn(u'HTTPError opening url=%r: %s %s' %
-                          (url, e.code, e.msg), error=e)
-    except urllib2.URLError, e:
-        # URLError like mailto: or other IO errors, e can be raised
-        cssutils.log.warn(u'URLError, %s' % e.reason, error=e)
-    else:
-        if res:
-            mimeType, encoding = encutils.getHTTPInfo(res)
-            if mimeType != u'text/css':
-                cssutils.log.error(u'Expected "text/css" mime type for url=%s but found: %r' %
-                                  (url, mimeType), error=ValueError)
-            return encoding, res.read()
-
 def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
     """
     Read cssText from url and decode it using all relevant methods (HTTP
-    header, BOM, @charset). Returns 
-    
-    - encoding used to decode text (which is needed to set encoding of 
+    header, BOM, @charset). Returns
+
+    - encoding used to decode text (which is needed to set encoding of
       stylesheet properly)
     - type of encoding (how it was retrieved, see list below)
     - decodedCssText
 
     ``fetcher``
-        see cssutils.registerFetchUrl for details
+        see cssutils.CSSParser.setFetcher for details
     ``overrideEncoding``
         If given this encoding is used and all other encoding information is
         ignored (HTTP, BOM etc)
     ``parentEncoding``
-        Encoding of parent stylesheet (while e.g. reading @import references sheets)
-        or document if available.
+        Encoding of parent stylesheet (while e.g. reading @import references
+        sheets) or document if available.
 
     Priority or encoding information
     --------------------------------
     **cssutils only**: 0. overrideEncoding
 
-    1. An HTTP "charset" parameter in a "Content-Type" field (or similar parameters in other protocols)
+    1. An HTTP "charset" parameter in a "Content-Type" field (or similar
+       parameters in other protocols)
     2. BOM and/or @charset (see below)
     3. <link charset=""> or other metadata from the linking mechanism (if any)
     4. charset of referring style sheet or document (if any)
@@ -780,9 +887,10 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
 
     """
     enctype = None
-    
+
     if not fetcher:
         fetcher = _defaultFetcher
+
     r = fetcher(url)
     if r and len(r) == 2 and r[1] is not None:
         httpEncoding, content = r
@@ -791,28 +899,42 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
             enctype = 0 # 0. override encoding
             encoding = overrideEncoding
         elif httpEncoding:
-            enctype = 1 # 1. HTTP            
+            enctype = 1 # 1. HTTP
             encoding = httpEncoding
         else:
-            # check content            
-            contentEncoding, explicit = cssutils.codec.detectencoding_str(content)
+            # BOM or @charset
+            if isinstance(content, unicode):
+                contentEncoding, explicit = codec.detectencoding_unicode(content)
+            else:
+                contentEncoding, explicit = codec.detectencoding_str(content)
+            
             if explicit:
                 enctype = 2 # 2. BOM/@charset: explicitly
                 encoding = contentEncoding
+                
             elif parentEncoding:
                 enctype = 4 # 4. parent stylesheet or document
                 # may also be None in which case 5. is used in next step anyway
                 encoding = parentEncoding
+                
             else:
                 enctype = 5 # 5. assume UTF-8
                 encoding = 'utf-8'
-        
-        try:
-            # encoding may still be wrong if encoding *is lying*!
-            decodedCssText = codecs.lookup("css")[1](content, encoding=encoding)[0]
-        except UnicodeDecodeError, e:
-            cssutils.log.warn(e, neverraise=True)
-            decodedCssText = None
+
+        if isinstance(content, unicode):
+            decodedCssText = content
+        else:
+            try:
+                # encoding may still be wrong if encoding *is lying*!
+                try:
+                    decodedCssText = codecs.lookup("css")[1](content, encoding=encoding)[0]
+                except AttributeError, ae:
+                    # at least in GAE
+                    decodedCssText = content.decode(encoding if encoding else 'utf-8')
+                    
+            except UnicodeDecodeError, e:
+                log.warn(e, neverraise=True)
+                decodedCssText = None
 
         return encoding, enctype, decodedCssText
     else:
